local _, addonTable = ...
local zc = addonTable.zc
local L = Merchant.Locales.GetString

--------------------------------------------------------------------------------

function Merchant.UI.StockpileTableScrollRefresh(value)
  local frame = Merchant.UI.StockpileTableScrollFrameContent
  local scrollFrameHeight = Merchant.UI.StockpileTableScrollFrame:GetHeight()

  local contentHeight = frame:GetHeight()
  local offset
  if scrollFrameHeight > contentHeight then
    offset = 0
  else
    offset = math.floor((contentHeight - scrollFrameHeight) / 1000.0 * value)
    offset = offset - offset % 35
  end
  frame:ClearAllPoints()
  frame:SetPoint("TOPLEFT", 0, offset)
  frame:SetPoint("TOPRIGHT", 0, offset)

  Merchant.UI.StockpileTableScrollValue = value
end

--------------------------------------------------------------------------------

function Merchant.UI.TestStockpileOrder(label, compareFunc)
  local stockpile = Merchant_Vars.Accounts[Merchant.RealmName].Stockpile[Merchant.UI.StockpileSelectedCharacter]
  if #stockpile > 1 then
    local previous = stockpile[1][label]
    for i = 2, #stockpile do
      local current = stockpile[i][label]
      if compareFunc(previous, current) == false then
        return false
      end
      previous = current
    end
  end
  return true
end

--------------------------------------------------------------------------------

Merchant.UI.StockpileSortFuncs = {
  ItemNameAsc = function(a, b)
    return a.ItemName:upper() < b.ItemName:upper()
  end,
  ItemNameDesc = function(a, b)
    return a.ItemName:upper() > b.ItemName:upper()
  end,
  ItemCountAsc = function(a, b)
    return a.ItemCount < b.ItemCount
  end,
  ItemCountDesc = function(a, b)
    return a.ItemCount> b.ItemCount
  end,
  ValueAsc = function(a, b)
    return a.Value < b.Value
  end,
  ValueDesc = function(a, b)
    return a.Value> b.Value
  end,
  TotalAsc = function(a, b)
    return a.Total < b.Total
  end,
  TotalDesc = function(a, b)
    return a.Total > b.Total
  end
}

--------------------------------------------------------------------------------

function Merchant.UI.SortStockpile(frame)
  local test = false
  if Merchant.UI.SortBy == nil then
    Merchant.UI.SortBy = frame.Label
    Merchant.UI.SortOrder = "Asc"
  elseif Merchant.UI.SortBy == frame.Label then
    if Merchant.UI.SortOrder == "Desc" then
      Merchant.UI.SortOrder = "Asc"
    else
      Merchant.UI.SortOrder = "Desc"
    end
  else
    Merchant.UI.SortBy = frame.Label
    Merchant.UI.SortOrder = "Asc"
  end

  local doSort = true

  local areAsc = Merchant.UI.TestStockpileOrder(Merchant.UI.SortBy, function(a, b) return a <= b end)
  local areDesc = Merchant.UI.TestStockpileOrder(Merchant.UI.SortBy, function(a, b) return a >= b end)

  if areAsc == true then
    Merchant.UI.SortOrder = "Desc"
  end

  if areDesc == true then
    Merchant.UI.SortOrder = "Asc"
  end

  if areAsc == true and areDesc == true then
    doSort = false
  end

  if doSort == true then
    table.sort(Merchant_Vars.Accounts[Merchant.RealmName].Stockpile[Merchant.UI.StockpileSelectedCharacter], Merchant.UI.StockpileSortFuncs[Merchant.UI.SortBy .. Merchant.UI.SortOrder])
    Merchant.UI.DrawStockpileTable(Merchant.UI.StockpileSelectedCharacter)
  end
end

--------------------------------------------------------------------------------

function Merchant.UI.EmptyStockpileTable()
  for key, row in pairs(Merchant.UI.StockpileRows) do
    row.ItemTexture:SetTexture("")
    row.ItemNameText:SetText("")
    row.QuantityText:SetText("")
    row.ValueText:SetText("")
    row.TotalText:SetText("")
  end
  Merchant.UI.StockpileRowCnt = 1
end

--------------------------------------------------------------------------------

function Merchant.UI.AddStockpileRow(stock)
  if #Merchant.UI.StockpileRows < Merchant.UI.StockpileRowCnt then
    local frame = Merchant.UI.StockpileTableScrollFrameContent
    local x = 0
    local y = -10 - 35 * (Merchant.UI.StockpileRowCnt - 1)
    local iconFrame = CreateFrame("Frame", nil, frame)
    iconFrame:SetSize(32, 32)
    iconFrame:SetPoint("TOPLEFT", x, y + 10)
    local itemTexture = iconFrame:CreateTexture(nil, "FULLSCREEN_DIALOG")
    itemTexture:SetAllPoints()
    x = x + 32 + 10
    local itemNameText = frame:CreateFontString(nil, "FULLSCREEN_DIALOG", "GameFontNormal")
    itemNameText:SetPoint("TOPLEFT", x, y)
    x = -270
    local quantityText = frame:CreateFontString(nil, "FULLSCREEN_DIALOG", "GameFontNormal")
    quantityText:SetPoint("TOPRIGHT", x, y)
    x = x + 125
    local valueText = frame:CreateFontString(nil, "FULLSCREEN_DIALOG", "GameFontNormal")
    valueText:SetPoint("TOPRIGHT", x, y)
    x = x + 125
    local totalText = frame:CreateFontString(nil, "FULLSCREEN_DIALOG", "GameFontNormal")
    totalText:SetPoint("TOPRIGHT", x, y)
    table.insert(Merchant.UI.StockpileRows, {
      IconFrame = iconFrame,
      ItemTexture = itemTexture,
      ItemNameText = itemNameText,
      QuantityText = quantityText,
      ValueText = valueText,
      TotalText = totalText
    })
  end

  local row = Merchant.UI.StockpileRows[Merchant.UI.StockpileRowCnt]
  row.ItemTexture:SetTexture(GetItemIcon(stock.ItemID))
  row.ItemNameText:SetText("|cFF" .. Merchant.Constants.UI.QUALITY_COLORS[stock.Quality] .. stock.ItemName)

  local pixelLimit = 237
  local charLimit = 30
  local itemName = row.ItemNameText:GetText()
  local offset = 0

  local function SplitText(str, chrLimit, off, doOff)
    local leftPartoff = 0
    local rightPartoff = 0
    if doOff == true then
      if off % 2 == 0 then
         rightPartoff = 1
      else
         leftPartoff = 1
      end
    end
    local leftPart = str:sub(1, 10 + chrLimit / 2 + off + leftPartoff)
    local rightPart = str:sub(str:len() - 2 - chrLimit / 2 - off - rightPartoff, str:len())
    return leftPart, rightPart
  end

  local function ReduceText(widget, str, chrLimit, off, doOff)
    local leftPart, rightPart = SplitText(str, chrLimit, off, doOff)
    widget:SetText(leftPart .. "..." .. rightPart)
  end

  if row.ItemNameText:GetWidth() > pixelLimit then
    ReduceText(row.ItemNameText, itemName, charLimit, offset, false)
    if row.ItemNameText:GetWidth() < pixelLimit then
      local previous = row.ItemNameText:GetText()
      while true == true do
        ReduceText(row.ItemNameText, itemName, charLimit, offset, true)
        if row.ItemNameText:GetWidth() > pixelLimit then
          break
        end
        offset = offset + 1
        previous = row.ItemNameText:GetText()
      end
      row.ItemNameText:SetText(previous)
      local leftPart, rightPart = SplitText(itemName, charLimit, offset, false)
      local points = "..."
      while true == true do
        points = points .. "."
        row.ItemNameText:SetText(leftPart .. points .. rightPart)
        if row.ItemNameText:GetWidth() > pixelLimit then
          break
        end
        previous = row.ItemNameText:GetText()
      end
      row.ItemNameText:SetText(previous)
    end
  end

  row.QuantityText:SetText("|cFFFFFFFF" .. tostring(stock.ItemCount))

  if stock.Value ~= 0 then
    row.ValueText:SetText("|cFFFFFFFF" .. zc.priceToMoneyString(stock.Value, true))
    row.TotalText:SetText("|cFFFFFFFF" .. zc.priceToMoneyString(stock.Total, true))
  else
    row.ValueText:SetText("|cFFFFFFFF" .. "N/A")
    row.TotalText:SetText("|cFFFFFFFF" .. "N/A")
  end

  Merchant.UI.StockpileRowCnt = Merchant.UI.StockpileRowCnt + 1
  Merchant.UI.StockpileTableScrollFrameContent:SetHeight(Merchant.UI.StockpileRowCnt * 34)
end

--------------------------------------------------------------------------------

function Merchant.UI.DrawStockpileTable(charName, filter)
  Merchant.UI.EmptyStockpileTable()
  local stockpile = Merchant_Vars.Accounts[Merchant.RealmName].Stockpile[charName]
  if stockpile == nil then
    Merchant.Debug("error: no stockpile for " .. charName .. " character")
    return
  end
  for idx, stock in ipairs(stockpile) do
    stock.Value = Merchant_Vars.AHPrices[Merchant.RealmName][tostring(stock.ItemID)]
    if stock.Value == nil then
      stock.Value = 0
      stock.Total = 0
    else
      stock.Total = stock.Value * stock.ItemCount
    end
    Merchant.UI.AddStockpileRow(stock)
  end
  if #stockpile > 9 then
    Merchant.UI.StockpileTableScrollFrameSlider:Show()
  else
    Merchant.UI.StockpileTableScrollFrameSlider:Hide()
  end
end
